@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2002 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -54,6 +54,7 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 	   n->handler       = NULL; 
 	   n->request       = NULL;
 	   n->request_local = NULL;
+	   n->host          = NULL;
 	   n->header_sent   = 0;
 	   n->keep_alive    = 0;
 	   n->timeout       = time(NULL) + 15;
@@ -80,6 +81,11 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 		   cnt->request_local = NULL;
 	   }
 	   
+	   if (cnt->host) {
+		   free (cnt->host);
+		   cnt->host = NULL;
+	   }
+
 	   free (cnt);
 	   return ret_ok;
 }
@@ -114,6 +120,11 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 		   cnt->handler = NULL;
 	   }
 
+	   if (cnt->host) {
+		   free (cnt->host);
+		   cnt->host = NULL;
+	   }
+
 	   return ret_ok;
 }
 
@@ -350,10 +361,35 @@ cherokee_connection_step (cherokee_connection_t *cnt)
 }
 
 
+static inline ret_t
+get_host (cherokee_connection_t *cnt, 
+	  char                  *ptr) /* Header at the "Host:" position */
+{
+	char *end;
+	int   size;
+
+	end = strstr (ptr, "\r");
+	if (end == NULL) {
+		return ret_error;
+	}
+	
+	size = (end - ptr);
+
+	cnt->host = (char *) malloc (size);
+	return_if_fail (cnt->host!=NULL, ret_nomem);
+	cnt->host[size] = '\0';
+	
+	memcpy (cnt->host, ptr, size);
+
+	return ret_ok;
+}
+
+
 static inline ret_t
 get_uri (cherokee_connection_t *cnt)
 {
 	int begin, end, len;
+	char *ptr;
 	
 	/* Malformed header
 	 */
@@ -409,22 +445,41 @@ get_uri (cherokee_connection_t *cnt)
 	memcpy (cnt->request, cnt->buffer->buf + begin, len);
 	cnt->request[len] = '\0';
 	
-	cnt->request_local = (char *) malloc(len + 2);
-	memcpy (cnt->request_local+1, cnt->buffer->buf + begin, len);
-	cnt->request_local[0] = '.';
-	cnt->request_local[len+1] = '\0';
-
 
 	/* Decode %HEX URL encoding
 	 */
 	strdecode (cnt->request, cnt->request);
-	strdecode (cnt->request_local, cnt->request_local);
+
+
+	/* Look for "Host:"
+	 */
+	if ((ptr = strstr (cnt->buffer->buf, "Host: "))) {
+		ptr += 6;
+		get_host (cnt, ptr);
+	}
 
 	
 	return ret_ok;
 }
 
 
+ret_t 
+cherokee_connection_build_local_request (cherokee_connection_t *cnt, cherokee_virtual_server_t *vsrv)
+{
+	int len, len2;
+
+	len  = strlen (cnt->request);
+	len2 = strlen (vsrv->root);
+
+	cnt->request_local = (char *) malloc(len + len2 + 1);
+	memcpy (cnt->request_local, vsrv->root, len2);
+	memcpy (cnt->request_local + len2, cnt->request, len);
+	cnt->request_local[len+len2] = '\0';
+
+	return ret_ok;
+}
+
+
 static inline ret_t
 get_range (cherokee_connection_t *cnt, 
 	   char                  *ptr) /* Header at the "Range: bytes=" position */
@@ -562,7 +617,8 @@ cherokee_connection_parse_header (cherokee_connection_t *cnt)
 ret_t
 cherokee_connection_open_request (cherokee_connection_t *cnt)
 {
-	return cherokee_handler_init (cnt->handler, cnt->request_local);
+//	return cherokee_handler_init (cnt->handler, cnt->request_local);
+	return cherokee_handler_init (cnt->handler, cnt->request, cnt->request_local);
 }
 
 
