@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2002 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,15 +27,23 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "connection.h"
 #include "handler_common.h"
 #include "handler_file.h"
 #include "handler_dirlist.h"
 
 
 ret_t 
-cherokee_handler_common_new (cherokee_handler_t **hdl, char *request)
+cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt)
 {
-	struct stat info;
+	struct stat  info;
+	char        *request;
+
+	/* Yeah, yeah..
+	 * This is very ugly, I know.. ;(
+	 */
+	request = CONN(cnt)->request_local;
+
 
 	/* Request is a directory
 	 */
@@ -51,9 +59,13 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, char *request)
 		sprintf (new, "%s/index.html", request);
 		
 		if ((stat (new, &info) == 0) && S_ISREG(info.st_mode)) {
-			ret = cherokee_handler_file_new	(hdl, new);
+			/* Sucks again :-((
+			 */
+			free (CONN(cnt)->request_local);
+			CONN(cnt)->request_local = new;
+			return cherokee_handler_file_new (hdl, cnt);
 		} else {
-			ret = cherokee_handler_dirlist_new (hdl, request);
+			ret = cherokee_handler_dirlist_new (hdl, cnt);
 		} 
 		
 		free (new);
@@ -63,7 +75,7 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, char *request)
 
         /* File handler
 	 */
-	return cherokee_handler_file_new (hdl, request);
+	return cherokee_handler_file_new (hdl, cnt);
 }
 
 
