@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2002 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -65,7 +65,7 @@ cherokee_handler_dirlist_new  (cherokee_handler_t **hdl, void *cnt)
 
 
 ret_t
-cherokee_handler_dirlist_init (cherokee_handler_dirlist_t *n, char *request)
+cherokee_handler_dirlist_init (cherokee_handler_dirlist_t *n, char *request, char *request_local)
 {
 	n->request = request;
 	n->request_len = strlen(request);
@@ -88,7 +88,7 @@ cherokee_handler_dirlist_init (cherokee_handler_dirlist_t *n, char *request)
 	
 	/* Open dir
 	 */
-	n->dir = opendir (request);
+	n->dir = opendir (request_local);
 	if (n->dir == NULL) {
 		CONN(HANDLER(n)->connection)->error_code = http_not_found;
 		return ret_error;
