@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2002 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,64 +40,63 @@
 #include "connection.h"
 #include "log.h"
 
+static void purge_closed_connection       (cherokee_server_t *srv, cherokee_connection_t *conn);
+static void maybe_purge_closed_connection (cherokee_server_t *srv, cherokee_connection_t *conn);
+
+
 
 ret_t
 cherokee_server_new  (cherokee_server_t **srv)
 {
 	cherokee_server_t  *n;
 	ret_t               ret;
-	int                 on = 1;
 	
 	
 	/* Get memory
 	 */
 	n = (cherokee_server_t*) malloc (sizeof(cherokee_server_t));
 	return_if_fail (n!=NULL, ret_nomem);
+
 	
+	/* Server socket
+	 */
+	n->socket = -1;
+
 	
 	/* Lists
 	 */
 	INIT_LIST_HEAD(&n->active_list);
 	INIT_LIST_HEAD(&n->reuse_list);
-
-	
-	/* Create the socket
-	 */
-	n->socket = socket (AF_INET, SOCK_STREAM, 0);
-	return_if_fail (n->socket > 0, ret_error);
-	
+ 
 	
 	/* File descriptors poll
 	 */
 	cherokee_fdpoll_new(&n->fdpoll);
-	cherokee_fdpoll_add(n->fdpoll, n->socket, 0);
 
 	
 	/* Server config
 	 */
-	ret = cherokee_server_conf_new (&n->config);
-	if (ret != ret_ok) {
-		return ret;
-	}
-	n->config->server = n;
+	n->port           = 80;
+	n->log            = 0;
+	n->timeout        = 15;
+	n->keepalive      = 1;
+	n->mimetypes_file = NULL;
 
-	
-	/* Plugin table
+	n->user_orig      = getuid();
+	n->user           = n->user_orig;
+	n->group_orig     = getgid();
+	n->group          = n->group_orig;
+
+ 
+	/* Virtual servers table
 	 */
-	cherokee_plugin_table_new (&n->plugins);
-	return_if_fail (n->plugins!=NULL, ret_nomem);
-	
-	
-	/* Set 'close-on-exec'
-	 */
-	fcntl(n->socket, F_SETFD, FD_CLOEXEC);
-	
-	
-	/* To re-bind without wait to TIME_WAIT
-	 */
-	ret = setsockopt (n->socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
-	return_if_fail (ret == 0, ret_error);
-	
+	cherokee_table_new (&n->vservers);
+	return_if_fail (n->vservers!=NULL, ret_nomem);
+
+	cherokee_virtual_server_new (&n->vserver_default);
+	return_if_fail (n->vserver_default!=NULL, ret_nomem);
+		
+
 	/* Return the object
 	 */
 	*srv = n;
@@ -107,28 +106,26 @@ cherokee_server_new  (cherokee_server_t **srv)
 
 
 static inline ret_t
-running_user_work (cherokee_server_t *srv)
+change_execution_user (cherokee_server_t *srv)
 {
 	int error;
 
 	/* Change of group requested
 	 */
-	if (srv->config->group != srv->config->group_orig) {
-		error = setgid (srv->config->group);
+	if (srv->group != srv->group_orig) {
+		error = setgid (srv->group);
 		if (error) {
-			snprintf (gbl_buffer, gbl_buffer_size, "Can't change group: setgid(\"%d\") failed", srv->config->group);
-			PRINT_ERROR_STR (gbl_buffer);
+			PRINT_ERROR ("Can't change group: setgid(\"%d\") failed\n", srv->group);
 			return ret_error;
 		}
 	}
 
 	/* Change of user requested
 	 */
-	if (srv->config->user != srv->config->user_orig) {
-		error = setuid (srv->config->user);		
+	if (srv->user != srv->user_orig) {
+		error = setuid (srv->user);		
 		if (error) {
-			snprintf (gbl_buffer, gbl_buffer_size, "Can't change user: setuid(\"%d\") failed", srv->config->user);
-			PRINT_ERROR_STR (gbl_buffer);
+			PRINT_ERROR ("Can't change user: setuid(\"%d\") failed\n", srv->user);
 			return ret_error;
 		}			
 	}
@@ -136,34 +133,54 @@ running_user_work (cherokee_server_t *srv)
 	return ret_ok;
 }
 
-
-ret_t
-cherokee_server_init (cherokee_server_t *srv) 
-{   
-	ret_t               ret;
+static ret_t
+initialize_server_socket (cherokee_server_t *srv)
+{
+	ret_t ret;
+	int   on = 1;
 	int                 flags;
 	struct sockaddr_in  sockaddr;
 
-	/* Log
+
+	/* Create the socket
 	 */
-	if (srv->config->log) {
-		cherokee_server_log (srv, 1);
-	}
- 
+	srv->socket = socket (AF_INET, SOCK_STREAM, 0);
+	return_if_fail (srv->socket > 0, ret_error);
+
+
+	/* Add the socket to fdpoll
+	 */
+	cherokee_fdpoll_add(srv->fdpoll, srv->socket, 0);
+
+
+	/* Set 'close-on-exec'
+	 */
+	fcntl(srv->socket, F_SETFD, FD_CLOEXEC);
+	
+	
+	/* To re-bind without wait to TIME_WAIT
+	 */
+	ret = setsockopt (srv->socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
+	return_if_fail (ret == 0, ret_error);
+
+
 	/* Bind the socket
 	 */
-	sockaddr.sin_port        = htons (srv->config->port);
+	sockaddr.sin_port        = htons (srv->port);
 	sockaddr.sin_family      = AF_INET;
 	sockaddr.sin_addr.s_addr = INADDR_ANY;
 	
 	ret = bind (srv->socket, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
 	if (ret != 0) {
-		cherokee_log (LOG_ERR, "Can't bind() socket (port %d)\n", srv->config->port);
-		fprintf (stderr, "Can't bind() socket (port %d)\n", srv->config->port);
+		uid_t uid = getuid();
+		gid_t gid = getgid();
+
+		cherokee_log (LOG_ERR, "Can't bind() socket (port %d, uid %d, %gid)\n", srv->port, uid, gid);
+		fprintf (stderr, "Can't bind() socket (port %d, uid %d, gid %d)\n", srv->port, uid, gid);
 		return ret_error;
 	}
 
-	cherokee_log (LOG_INFO, "Listening on port %d", srv->config->port);
+	cherokee_log (LOG_INFO, "Listening on port %d", srv->port);
 
 	   
 	/* Set no-delay mode
@@ -183,10 +200,37 @@ cherokee_server_init (cherokee_server_t *srv)
 		return_if_fail (0, ret_error);
 	}
 
+	return ret_ok;
+}
+
+
+ret_t
+cherokee_server_init (cherokee_server_t *srv) 
+{   
+	ret_t ret;
+
+
+	/* Init the log system
+	 */
+	if (srv->log) {
+		cherokee_server_log (srv, 1);
+	}
+
+
+	/* If the server has a previous server socket opened, Eg:
+	 * because a SIGHUP, it shouldn't init the server socket.
+	 */
+	if (srv->socket == -1) {
+		ret = initialize_server_socket (srv);
+		if (ret != ret_ok) {
+			return ret;
+		}
+	}
+
 
 	/* Change the user
 	 */
-	ret = running_user_work (srv);
+	ret = change_execution_user (srv);
 	if (ret != ret_ok) {
 		return ret;
 	}
@@ -196,12 +240,67 @@ cherokee_server_init (cherokee_server_t *srv)
 }
 
 
+static inline void
+close_all_connections (cherokee_server_t *srv)
+{
+	ret_t  ret;
+	list_t *i, *tmp;
+
+
+	/* Close connectios and move to reusable list
+	 */
+	list_for_each_safe (i, tmp, (list_t*)&srv->active_list) {
+		purge_closed_connection (srv, CONN(i));
+	}
+}
+
+
+ret_t 
+cherokee_server_clean (cherokee_server_t *srv)
+{
+	ret_t ret;
+
+	/* Close all active connections
+	 */
+	close_all_connections (srv);
+
+
+	/* Clean the configuration
+	 */
+	if (srv->mimetypes_file != NULL) {
+		free (srv->mimetypes_file);
+		srv->mimetypes_file = NULL;
+	}
+
+	/* Clean the virtual servers table
+	 */
+	cherokee_virtual_server_clean (srv->vserver_default);
+ 
+	// TODO:
+	// Free each vserver
+	cherokee_table_free (srv->vservers);
+	cherokee_table_new (&srv->vservers);
+
+	return ret_ok;
+}
+
+
 ret_t
 cherokee_server_free (cherokee_server_t *srv)
 {
+	close (srv->socket);
+
 	cherokee_fdpoll_free (srv->fdpoll);
-	cherokee_plugin_table_free (srv->plugins);
-	cherokee_server_conf_free (srv->config);
+	srv->fdpoll = NULL;
+
+	cherokee_virtual_server_free (srv->vserver_default);
+	srv->vserver_default = NULL;
+
+	if (srv->mimetypes_file != NULL) {
+		free (srv->mimetypes_file);
+		srv->mimetypes_file = NULL;
+	}
+
 	
 	free (srv);
 	
@@ -214,17 +313,17 @@ cherokee_server_log (cherokee_server_t *srv, int active)
 {
 	/* Shutdown loggin
 	 */
-	if (srv->config->log && !active) {
-		srv->config->log = active;
+	if (srv->log && !active) {
+		srv->log = active;
 		cherokee_log (LOG_NOTICE, "Closing log");
 		return cherokee_log_close();
 	}
 	
 	/* Active logging
 	 */
-	if (active && !srv->config->log) {
+	if (active && !srv->log) {
 		ret_t ret;
-		srv->config->log = active;
+		srv->log = active;
 		ret = cherokee_log_init();
 		cherokee_log (LOG_NOTICE, "Starting log");
 		return ret;
@@ -259,7 +358,7 @@ look_for_new_connection (cherokee_server_t *srv)
 			new_connection = CONN(srv->reuse_list.next);
 			list_del ((list_t *)new_connection);
 
-			new_connection->timeout = srv->bogo_now + srv->config->timeout;
+			new_connection->timeout = srv->bogo_now + srv->timeout;
 		}
 
 		
@@ -323,7 +422,7 @@ maybe_purge_closed_connection (cherokee_server_t *srv, cherokee_connection_t *co
 	 */
 	if (conn->keep_alive) {
 		cherokee_connection_clean (conn);
-		conn->timeout = srv->bogo_now + srv->config->timeout;
+		conn->timeout = srv->bogo_now + srv->timeout;
 		conn_set_mode (srv, conn, stat_reading);
 		return;
 	}
@@ -360,7 +459,7 @@ process_active_connections (cherokee_server_t *srv)
 
 		/* The connection has work, so..
 		 */
-		conn->timeout = srv->bogo_now + srv->config->timeout;
+		conn->timeout = srv->bogo_now + srv->timeout;
 
 		
 		/* Reading
@@ -385,6 +484,7 @@ process_active_connections (cherokee_server_t *srv)
 			/* May it already has the full header
 			 */
 			if (cherokee_connection_eoh (conn)) {
+				cherokee_virtual_server_t *vserver = NULL;
 
 				/* Turn the connection in write mode
 				 */
@@ -403,12 +503,28 @@ process_active_connections (cherokee_server_t *srv)
 
 				/* Server's "Keep-Alive" can be turned "Off"
 				 */
-				conn->keep_alive &= srv->config->keepalive;
+				conn->keep_alive &= srv->keepalive;
+
+				/* Get the virtual host
+				 */
+				if (conn->host) {
+					vserver = cherokee_table_get (srv->vservers, conn->host);
+				}
+				if (vserver == NULL) {
+					vserver = srv->vserver_default;
+				}
+
+				/* Build local request string
+				 */
+				ret = cherokee_connection_build_local_request (conn, vserver);
+				if (ret != ret_ok) {
+					continue;
+				}
 
-				
 				/* Create the handler
 				 */
-				ret = cherokee_connection_create_handler (conn, srv->plugins);
+				ret = cherokee_connection_create_handler (conn, vserver->plugins);
+				
 				if (ret != ret_ok) {
 					continue;
 				}				
@@ -496,7 +612,7 @@ process_active_connections (cherokee_server_t *srv)
 }
 
 
-
+/*
 ret_t 
 cherokee_server_set (cherokee_server_t *srv, char *dir, char *handler)
 {
@@ -517,7 +633,7 @@ cherokee_server_set (cherokee_server_t *srv, char *dir, char *handler)
 
 	   return ret;
 }
-
+*/
 
 void
 cherokee_server_step (cherokee_server_t *srv)
@@ -543,3 +659,33 @@ cherokee_server_step (cherokee_server_t *srv)
 }
 
 
+ret_t 
+cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
+{
+	int error;
+	extern FILE *yyin;
+
+	/* Set the file to read
+	 */
+	yyin = fopen (filename, "r");
+	return_if_fail (yyin!=NULL, ret_file_not_found);
+
+	/* Cooooome on :-)
+	 */
+	error = yyparse ((void *)srv);
+
+	if (yyin != NULL)
+		fclose (yyin);
+
+	if (error != 0)
+		return ret_error;
+
+	return ret_ok;
+}
+
+
+ret_t 
+cherokee_server_read_config_string (cherokee_server_t *srv, char *config_string)
+{
+	return ret_ok;
+}
